//*****************************************************************************
//*
//*
//*		FtpWindow.h
//*
//*
//*****************************************************************************
#ifndef		__FTPWINDOW_H__
#define		__FTPWINDOW_H__

#if _MSC_VER > 1000
#pragma once
#endif

#define 	CFW_CONN_INFO		0x2000
#define 	CFW_PATH_CHANGE		0x2001
#define 	CFW_MAX_PATHS		32
#define 	CFW_TIMEOUT			30

#ifndef 	CFW_MAX_PATH
#define 	CFW_MAX_PATH		512	
#endif 	

// Konstanden fr die Suche
#define 	CFW_FIND_ADDFILES	0x8000
#define 	CFW_FIND_ADDDIR		0x4000
#define 	CFW_FIND_SUBDIR		0x2000
#define 	CFW_FIND_NOLINKS	0x1000
#define 	CFW_FIND_ALL		0x0000
#define 	CFW_FIND_RDONLY		0x0100
#define 	CFW_FIND_RDWR		0x0200
#define 	CFW_FIND_TIMEALL	0x0000
#define 	CFW_FIND_CREATEPRE	0x0010
#define 	CFW_FIND_CREATEPAST	0x0020
#define 	CFW_FIND_WRITEPRE	0x0030
#define 	CFW_FIND_WRITEPAST	0x0040

// Konstanden fr m_iFlags
#define 	CFW_DEFAULT			0x0107			// Alle Spalten auer Suffix anzeigen
#define 	CFW_SIZE			0x0001			// Dateigre anzeigen
#define 	CFW_TYPE			0x0002			// Dateityp anzeigen
#define 	CFW_TIME			0x0004			// nderungszeit anzeigen
#define 	CFW_CREATE			0x0008			// Erstellungszeit anzeigen
#define 	CFW_ATTRIBUTE		0x0010			// Dateiattribute
#define		CFW_SETCOLUMN		0x0100			// Die Spalten mit rechten Mausklick einblenden
#define		CFW_NOLISTLOAD		0x0200			// Liste der ComboBox nicht laden



#include	<Afx.h>
#include	<AfxMt.h>
#include	<AfxOle.h>
#include	<AfxMacros.h>
#include	"ProgressDlg.h"
#include	"TreeListCtrl.h"
#include	"FtpTools/FtpTools.h"

class		CFtpBase;
class		CFtpWindow;
struct		CopyStream;

typedef struct
	{
	NMHDR	sHdr;								// Notify Header
	UINT	uConnId;							// 0=Fehler 1=Verbindung ok  2=Diskonnect
	UINT	uErrorId;							// Windows Fehler ID
	LPCTSTR	pErrorText;							// Fehler als Text (NULL=kein Fehler)
	LPCTSTR	pShortPath;							// Kurzer Namenspfad mit Server
	LPCTSTR	pConnPath;							// Aktueller Pfad der Verbindung	
	}CFT_CONN_NOTIFY,*LPCFT_CONN_NOTIFY;


DECLARE_DROP_FUNC_MAIN(CFtpWindow);

//*****************************************************************************
//*
//*		Klasse CFtpWindow
//*
//*****************************************************************************
class CFtpWindow : public CWnd
{
public:

	CFtpWindow();
   ~CFtpWindow();

	BOOL			Create(const RECT &sRect,CWnd *pParentWnd,UINT nID,LPCTSTR pRegName=0,UINT uFlags=CFW_DEFAULT);
	int 			FindFiles(CStringChain &sList,LPCTSTR pPath,int iAttrib=CFW_FIND_ADDFILES|CFW_FIND_ALL|CFW_FIND_TIMEALL,LPCTSTR pPlus=0,LPCTSTR pMinus=0,time_t iTime=-1);
	int 			GetPath			 (LPTSTR pPath,int iSize,int iMode=0);
	void			SetModeSort      (int  iMode,int  iDir=-1);
	void			GetModeSort      (int &iMode,int &iDir);
	void  			GetModeFolders   (int &iMode);
	void			SetModeFolders   (BOOL bOn);
	void			SetModeFullRow   (BOOL bOn);
	void			SetModeAlternate (BOOL bOn);
	void			GetConnectionName(CString &sText);
	CWnd	       *SetFocus		 (int  iMode=0);
	DWORD			SetFlags		 (DWORD uFlags,DWORD uMask=0xFFFFFFFF);
	DWORD			GetFlags		 ();
	DWORD			GetLastConnErrId ();	
	LPCTSTR			GetLastConnError ();	
	LPCTSTR			GetShortPath     ();	
	

	BOOL			CopyFtpToFtp     (CFtpWindow *pTarget);
	BOOL			CopyFtpToFtp     (CFtpWindow *pTarget,LPCTSTR pPath,BOOL bIgnoreLinks=FALSE);
	BOOL			SetConnection    (CFtpBase *pConnection);
	BOOL			SetPath          (LPCTSTR pPath,BOOL bAddHistory=TRUE,BOOL bMustExist=TRUE);
	BOOL			InsertEntry      (LPCTSTR pName=0,BOOL bRename=TRUE,BOOL bFile=FALSE);
	BOOL			Connect          (int iMode,LPCTSTR pServer,LPCTSTR pUser=0,LPCTSTR pPasswd=0,LPCTSTR pPath=0,BOOL bAutoConn=1,BOOL bRetry=1,BOOL bPassive=0,BOOL bLinkResolve=0,BOOL bIgnoreLinks=0,BOOL bReadOnly=0,UINT uTimeout=CFW_TIMEOUT,UINT uPort=0,BOOL bThread=FALSE);
	BOOL			Connect          (LPCTSTR pUrl,LPCTSTR pName=0,BOOL bAutoConn=1,BOOL bRetry=1,BOOL bPassive=0,BOOL bLinkResolve=0,BOOL bIgnoreLinks=0,BOOL bReadOnly=0,UINT uTimeout=CFW_TIMEOUT,BOOL bThread=FALSE);
	BOOL			Connect          (int iMode=0,BOOL bThread=FALSE);
	BOOL			SortItems        (int iMode=-1,int iDir=-1);
	BOOL			HexEditor        (LPCTSTR pPath=NULL);
	BOOL			Editor           (LPCTSTR pPath=NULL);
	BOOL			UpdateFolder     (HTREEITEM hItem);
	BOOL			UpdateList       ();
	BOOL			Disconnect       ();
	BOOL			IsReadOnly       ();
	BOOL			IsConnected      ();
	BOOL			IsSelcted        ();
	BOOL			IsFolder         ();
	BOOL			IsCombo          ();
	BOOL			IsFile           ();
	BOOL			IsRoot           ();


protected:

	BOOL		  	GetSelection     (CProgressDlg *pDlg=0);
	BOOL			GetSelectionDlg  ();
	void		  	SetSelText       ();
	void 		  	SetComboText     (LPCTSTR		pPath);
	void			PostConnNotify   (unsigned      uConnId);
	void			GetConnText	     (CFtpBase     *pConnection,CString &sText);
	void 		  	InsertCmd	     (LPCTSTR		pPath ,int iCmd,LPCTSTR pTarget=0);
	BOOL		  	AddToHistory     (LPCTSTR		m_cCurrentPath);
	BOOL			LoadList	     (LPCTSTR		pName,LPCTSTR pPath=0);
	UINT			GetPathHash	     (LPTSTR		pName,int iAddSlash=0);
	BOOL		  	GetFolder        (HTREEITEM     hItem=0);
	BOOL		  	SetAttributes    (HTREEITEM     hItem);
	BOOL		  	Rename		     (HTREEITEM     hItem);
	BOOL		  	IsFile	         (HTREEITEM     hItem);
	BOOL		  	IsFolder	     (HTREEITEM     hItem);
	BOOL		  	OpenFolder       (HTREEITEM     hItem);
	BOOL		  	StartShell       (HTREEITEM     hItem);
	BOOL		  	StartEditor      (HTREEITEM     hItem);
	BOOL		  	StartHexEditor   (HTREEITEM     hItem);
	int 			GetFullPath	     (HTREEITEM     hItem,LPTSTR  pPath,int iSize,int iMode=0);
	int 			GetDirPath	     (HTREEITEM     hItem,LPTSTR  pPath,int iSize);
	void			InsertDirectory  (HTREEITEM     hItem,LPCTSTR pName,BOOL bExpand=FALSE); 
	BOOL			RenameDirectory  (HTREEITEM     hItem,LPCTSTR pOldName,LPCTSTR pNewName); 
	BOOL			ConnectThread    (int           iMode,LPCTSTR pServer,LPCTSTR pUser,LPCTSTR pPasswd,LPCTSTR pPath,BOOL bAutoConn,BOOL bRetry,BOOL bPassive,BOOL bLinkResolve,BOOL bIgnoreLinks,BOOL bReadOnly,UINT uTimeout,UINT uPort,BOOL bThread=FALSE);
	int			  	ExtractSingle    (HTREEITEM     hItem ,LPTSTR  pTarget=NULL,int iMax=0,int iEdit=FALSE);
	int				ExtractSingle    (LPCTSTR       pPath ,LPTSTR  pTarget=NULL,int iMax=0,int iEdit=FALSE);
	int			  	ThreadMessage    (LPCSTR	    pText ,LPTSTR  pTitle=_T("Fehler"),int iMode=MB_OK|MB_ICONERROR);
	LRESULT			InsertPath	     (LPTSTR        pPath,unsigned uMode);
	HTREEITEM		FindDirectory    (LPCTSTR       pName);
	static LPCTSTR	ErrorToText      (unsigned      uError,LPCTSTR pDefault);
	static BOOL		WaitForCriticalSection(LPCRITICAL_SECTION pSection,unsigned uTimeout);


	static	int	  	CALLBACK ExecuteCb(HWND hWnd ,LPVOID    pParam,UINT uCommand);
	static	LRESULT CALLBACK ComboProc(HWND hWnd ,UINT      uMsg ,WPARAM  wParam   ,LPARAM lParam);
	static	int	  	CALLBACK InsertCb (HWND hWnd ,HTREEITEM hItem,LPCTSTR pTextItem,LPCTSTR pTextInsert,LPARAM lParamItem,LPARAM lParamInsert);
	static	int	  	CALLBACK InsertDir(HWND hWnd ,HTREEITEM hItem,LPCTSTR pTextItem,LPCTSTR pTextInsert,LPARAM lParamItem,LPARAM lParamInsert);
	static	int   	CALLBACK SortProc0(HWND hTree,HTREEITEM hItem1,HTREEITEM hItem2,LPARAM lParam1,LPARAM lParam2,LPARAM lParamSort);
	static	int   	CALLBACK SortProc1(HWND hTree,HTREEITEM hItem1,HTREEITEM hItem2,LPARAM lParam1,LPARAM lParam2,LPARAM lParamSort);
	static	int   	CALLBACK SortProc2(HWND hTree,HTREEITEM hItem1,HTREEITEM hItem2,LPARAM lParam1,LPARAM lParam2,LPARAM lParamSort);
	static	int   	CALLBACK SortProc3(HWND hTree,HTREEITEM hItem1,HTREEITEM hItem2,LPARAM lParam1,LPARAM lParam2,LPARAM lParamSort);
	static	int   	CALLBACK SortProc4(HWND hTree,HTREEITEM hItem1,HTREEITEM hItem2,LPARAM lParam1,LPARAM lParam2,LPARAM lParamSort);
	static	int   	CALLBACK SortProc5(HWND hTree,HTREEITEM hItem1,HTREEITEM hItem2,LPARAM lParam1,LPARAM lParam2,LPARAM lParamSort);



	//{{AFX_MSG(CFtpWindow)
	afx_msg BOOL	OnSetCursor(CWnd *pWnd,UINT nHitTest,UINT uMsg);
	afx_msg void	OnKeyDown(UINT nChar,UINT nRepCnt,UINT nFlags);
	afx_msg void 	OnContextMenu(CWnd *pWnd ,CPoint sPoint);
	afx_msg void 	OnLButtonDown(UINT nFlags,CPoint sPoint);
	afx_msg void 	OnSize (UINT nType,int iSizeX,int iSizeY);
	afx_msg void 	OnTimer(UINT_PTR nIdEvent);
	afx_msg void 	OnSetFocus(CWnd *pWnd);
	afx_msg void 	OnShowCol(UINT nId);
	afx_msg void 	OnCut();
	afx_msg void 	OnCopy();
	afx_msg void 	OnPaste();
	afx_msg void 	OnPaint();
	afx_msg void 	OnPathUp();
	afx_msg void 	OnRename();
	afx_msg void 	OnDelete();
	afx_msg void 	OnEditor();
	afx_msg void 	OnTreeOn();
	afx_msg void 	OnConnect();
	afx_msg void 	OnNewFile();
	afx_msg void 	OnDestroy();
	afx_msg void 	OnFileFind();
	afx_msg void 	OnPathNext();
	afx_msg void 	OnPathPrev();
	afx_msg void 	OnEnumDone();
	afx_msg void 	OnListDone();
	afx_msg void 	OnDropDone();
	afx_msg void 	OnDropThis();
	afx_msg void 	OnCopyPath();
	afx_msg void 	OnHexEditor();
	afx_msg void 	OnSelectAll();
	afx_msg void 	OnDoConnect();
	afx_msg void 	OnNewFolder();
	afx_msg void 	OnSelChange();
	afx_msg void 	OnSetCbFocus();
	afx_msg void 	OnOpenFolder();
	afx_msg void 	OnDisconnect();
	afx_msg void 	OnCursorWait();
	afx_msg void 	OnCursorArrow();
	afx_msg void 	OnSetAttributes();
	afx_msg void 	OnSetAttributesUI(CCmdUI *pCmdUI);
	afx_msg void 	OnOpenFolderUI   (CCmdUI *pCmdUI);
	afx_msg void 	OnHexEditorUI    (CCmdUI *pCmdUI);
	afx_msg void 	OnSelectAllUI    (CCmdUI *pCmdUI);
	afx_msg void 	OnPathNextUI     (CCmdUI *pCmdUI);
	afx_msg void 	OnPathPrevUI     (CCmdUI *pCmdUI);
	afx_msg void 	OnExtractUI      (CCmdUI *pCmdUI);
	afx_msg void 	OnDeleteUI       (CCmdUI *pCmdUI);
	afx_msg void 	OnEditorUI       (CCmdUI *pCmdUI);
	afx_msg void 	OnRenameUI       (CCmdUI *pCmdUI);
	afx_msg void 	OnPasteUI        (CCmdUI *pCmdUI);
	afx_msg void 	OnCopyUI         (CCmdUI *pCmdUI);
	afx_msg void 	OnCutUI          (CCmdUI *pCmdUI);
	//}}AFX_MSG

	afx_msg void	OnPathSelect	 (UINT   uId);
	afx_msg LRESULT OnMessageBox     (WPARAM wParam,LPARAM   lParam );
	afx_msg LRESULT OnChangeIcon     (WPARAM wParam,LPARAM   lParam );
	afx_msg LRESULT OnConnectDone    (WPARAM wParam,LPARAM   lParam );
	afx_msg void	OnColumnRClick	 (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnBeginFolderEdit(NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnBeginLabelEdit (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnEndLabelEdit   (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnKeyDownCombo   (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnKeyDownFile    (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnKeyDownTree    (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnGetDispInfo    (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnColumnClick    (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnDoubleClick    (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnSelChanged     (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnDirChanged	 (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnBeginDrag      (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnExpanding      (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnButtonUp		 (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnSetFocus	     (NMHDR *pNmHdr,LRESULT *pResult);
	afx_msg void	OnReturn         (NMHDR *pNmHdr,LRESULT *pResult);


	DECLARE_MESSAGE_MAP()
	DECLARE_DROP_FUNC2(CFtpWindow)

	typedef struct 
		{
		HANDLE			m_hEventProc;				// Freigabe fr externen Aufruf
		HANDLE			m_hEventCall;				// Freigabe fr internen Aufruf
	   CRITICAL_SECTION	m_sLock;					// Lokale Semaphore
		int volatile	m_iEnd;						// Wurde alles gelesen
		int volatile	m_iDone;					// Wurde alles verarbeitet
		int volatile	m_iBytes;					// Anzahl der Byte im Puffer
		__int64			m_iPosRd;					// Gelesene Bytes
		__int64			m_iPosWr;					// Geschriebene Bytes
		unsigned		m_uBufferSize;				// Ist die Puffergre in Byte
		BYTE			m_aFirst [4];				// Die ersten bertragenen Byte
		BYTE			m_aBuffer[4];				// bertragungspuffer
		}CopyStream;

	#ifdef	_DEBUG
	char				m_cLockPoint[256];			// Frs Debuggen von LOCK-Fehlern
	#endif 	

	CString				m_aPaths[CFW_MAX_PATHS];	// Pfad-History
	int					m_iPathStart;				// Erster Pfad in History
	int					m_iPathEnd;					// Letzter Pfad in History
	int					m_iPathPos;					// Aktueller Pfad in History
	int					m_iPathOk;					// Gibt es Pfade in der History
	int					m_iWindowId;				// Ist die ID des Fensters
	int					m_iWindowX;					// Ist die X-Fenstergre
	int					m_iWindowY;					// Ist die X-Fenstergre
	CButton				m_cBtPrev;					// Zurck Taste	
	CButton				m_cBtNext;					// Vortaste Taste	
	CButton				m_cBtConnect;				// Verbinden Taste	
	CButton				m_cBtDisconnect;			// Trennen Taste	
	CButton				m_cBtFolderUp;				// Eine Ebenehher
	CComboBox			m_cComboBox;				// Aktueller Pfad
	CFtpBase		   *m_pFtpConnection;			// Ist die FTP-Verbindung
	CFtpBase		   *m_pFtpTarget;				// Ist die FTP-Verbindung vom Ziel
	CString				m_sListName;				// Registry-Name fr die ComboListe
	CString				m_sRegName;					// Registry-Name fr das Fenster
	CString				m_sConnectPath;				// Pfad von der neuen Verbindung
	CRITICAL_SECTION	m_sMainSynch;				// CriticalSection fr das Fenster
	CRITICAL_SECTION	m_sListSynch;				// CriticalSection fr die Editiersliste
	int					m_iTreeWidth;				// Breite des Verzeichnisfensters
	int					m_iTreePos;					// Aktuelle Breite des Verzeichnissfesters
	UINT				m_uFlags;					// Diverse Flags (CFW_???)	
	BOOL				m_bTreeOn;					// Ist das Verzeichnissfester an
	BOOL				m_bWaitCursor;				// Ist das Warte-Cursor aktiv
	BOOL				m_bQueryForSave;			// Speichern abfrage bei nderungen
	BOOL				m_bDoConnect;				// Wird gerade eine Verbindung aufgebaut
	BOOL				m_bOkConnect;				// Ergebnis des Verbindungsaufbaus
	BOOL				m_bIgnoreLinks;				// Soft-Links beim Kopieren ignorieren
	BOOL				m_bNoConnError;				// Eine Verbindungfehler anzeigen
	BOOL volatile		m_bBreakAction;				// Den aktuellen Vorgang abbrechen
	BOOL volatile		m_bListStoped;				// Wurde die letzte Dateisuche abgebrochen
	BOOL volatile		m_bListBreak;				// Die aktuelle Dateisuche abbrechen
	BOOL volatile		m_bListOn;					// Ist der Thread fr die Dateisuche an
	TCHAR				m_cCurrentPath[CFW_MAX_PATH];//Aktueller FTP Pfad
	TCHAR				m_cShortPath[128];			// Anzeigename fr die Verbindung
	TCHAR				m_cMainPath[CFW_MAX_PATH];	// Anfangspfad der in der Ansicht nicht glscht werden kann
	TCHAR				m_cListPath[CFW_MAX_PATH];	// Pfad fr den List-Thread
	TCHAR				m_cEditType[16];			// Typ einer Datei vor dem Editieren
	HTREEITEM			m_hSelectItem;				// Wurde ein neuer Folder vorab gewhlt
	HTREEITEM			m_hListItem;				// Eintrag in dem die Verzeichnisse gelisted werden
	int					m_iListMode;				// 0=alles 1=nur Verzeichnisse
	int					m_iListTick;				// Ticker fr Auswahl im Verzeichnissfenster
	int					m_iDirIcon;					// Icon-Nummer fr Verzeichnisse
	int					m_iNewIcon;					// Icon-Nummer fr neue Dateien
	int					m_iSelIcon;					// Icon-Nummer fr Verzeichnisse (geffnet)
	TCHAR				m_cSetDropDown[128];		// DropDown Liste nach dem Verbinden laden
	TCHAR				m_cSelectBase [CFW_MAX_PATH];//Basisverzeichnis fr Selcet
	TCHAR				m_cSetDropPath[CFW_MAX_PATH];//DropDown Liste nach dem Verbinden laden
	CStringChain		m_sSelectList;				// Liste aller ausgewhlten Dateien
	int					m_iSelectLen;				// Lnge des Selcet Basisverzeichnises
	HWND				m_hEditCtrl;				// Wird ein Eintrag etitiert
	BOOL				m_bReadOnly;				// Hat die Verbindung einen Schreibschutz
	BOOL				m_bUpdateFlag;				// Auch Verzeichnis aktualisieren
	int					m_iFocusMode;				// 0=Combo 1=TreeList 2=FileList
	int					m_iFocusLock;				// Ist die Focus-Weiterleitung gesperrt
	CWnd				m_cComboEdit;				// Edit-Fenster der Combobox
	HCURSOR				m_hCursor;					// Aktuelles Cursor-Handle (NULL default)

	TCHAR				m_cTempDir [CFW_MAX_PATH];	// Ist der tempore Pfad samt Backslash und ProzessID
	TCHAR				m_cTempBase[CFW_MAX_PATH];	// Ist der tempore Pfad samt Backslash
	int					m_iTempDirSize;				// Gre des Temorren Pfades
	DWORD				m_uPathMode;				// Pfadmodus abfragen (0=ftp:// 1=//... 2=D:\...)
	BYTE				m_aFontSize[256];			// Buchstabennreiten fr den ProgressDlg
	UINT volatile		m_uRedrawCount;				// Anzahl der bereits eingefgten Eintrge
	CTreeListCtrlEx		m_cFileList;				// Fenster in das Archive angezeigt wird
	CTreeListCtrlEx		m_cTreeList;				// Fenster in das Verzeichnis angezeigt wird
	CStringChain		m_sTempFiles;				// Sind Temporre Dateien die gelscht werden mssen
	CStringChain		m_sEditList;				// Liste aller gerade editierten Dateien
	CImageList			m_cImageList;				// Spezial-Icons fr die Fenster
	int					m_iAlterante;				// Alternierente Farben in der Liste
	int					m_iSortMode;				// 0=Name 1=Typ 2=Datum 3=Gre 
	int					m_iFullRow;					// Die ganze Zeile auswhlen
	int					m_iColMark;					// Welche Spalte ist markiert
	int					m_iSortDir;					// Aufwrts=1 oder Abwrtssortieren=-1
	static int volatile	m_iRunning;					// Ist 1 sollange das Programm luft
	__int64				m_iProgressTemp;			// Temporrer Wert fr die Progress-Funktion
	__int64				m_iProgressStep;			// Gre der aktuellen Datei
	__int64				m_iProgressNow;				// Startwert fr die Progress-Funktion
	__int64				m_iProgressMax;				// Maximal Wert fr die Progress-Funktion
	int					m_iProgressMode;			// Modus der Progress-Funktion (0=nicht ndern 1=von Quelle 2=von Ziel 3=Null)
	CFT_CONN_NOTIFY		m_sConnNotify;				// Notify Nachricht fr Verbindungsaufbau
	HTREEITEM			m_hExecuteItem;				// Eintrag fr ExecuteCb
	HTREEITEM			m_hCurrentDir;				// Ist der Eintrag des aktuellen Verzeichnisses
	HTREEITEM			m_hExpandDir;				// Ist der Eintrag des automatisch aufgeklappt wurde
	LPVOID				m_pCancelPtr;				// Ist das Abbruch-Handle
	DWORD				m_dwLastError;				// Ist der letzte Fehler beim Verbindungsaufbau
	LPCTSTR				m_pLastError;				// Letzter Fehler als Text

	static CFtpWindow  *m_pDropThis;				// Zeiger auf das Fenster das einen Drop erhielt
	static CFtpWindow  *m_pDragSource;				// Zeiger auf das Fenster das einen Drag beginnt
	static HTREEITEM	m_hDragFolder;				// Verzeichniseintrag fr Drag (0 bei Dateifenster)
	static HTREEITEM	m_hDragLast;				// Verzeichniseintrag mit dem letzten Drag
	int			 		m_iDropDone;				// Letzter Modus

	static CopyStream  *m_pCopyStream;				// Aktueller Copy-Stream
	static CFtpBase	   *m_pCopySource;				// Aktuelle  Copy-Quelle
	static CFtpBase	   *m_pCopyTarget;				// Aktuelles Copy-Ziel

	CPoint		 		m_sDropPoint;				// Letzter Punkt
	CStringChain		m_sDropList;				// Liste mit den Dateinamen
	TCHAR				m_cDropPath[CFW_MAX_PATH];	// Ziel fr eine Drop-Operation
	int					m_iDropAction;				// 0=Nichts  1=Drop in Liste  2=Drop in Verzeichnis
	int					m_iDropMode;				// 0=Nichts  1=Drop in Liste  2=Drop in Verzeichnis
	HTREEITEM			m_hDropItem;				// Eintrag fr Drop Aktion
	HTREEITEM			m_hDropDone;				// Eintrag auf dem gedropt wurde

	LONG volatile		m_lThreadCount;				// Anzahl der laufenden Threads
	int					m_iSelTimer;				// Ist der Aktualisierungs-Timmer ein
	int					m_iSelLock;					// Ausgewhlten Dateien nicht ausgeben
	int					m_iSelCount;				// Anzahl der ausgewhlten Dateien
	__int64				m_iSelBytes;				// Anzahl der ausgewhlten Byte
	static TCHAR		m_cSelText[128];			// Text fr die ausgewhlten Dateien

	static UINT			m_uFormatFileDesc;			// Clipboard-Format-ID fr Dateiliste
	static UINT			m_uFormatFileData;			// Clipboard-Format-ID fr Dateidaten
	static LPCTSTR		m_aNames[];					// Namen fr Spalten

	static	int  _cdecl	SelProc			(CProgressDlg *pDlg);
	static	int  _cdecl	FindProc		(CProgressDlg *pDlg);
	static	int  _cdecl	CopyProc		(CProgressDlg *pDlg);
	static	int  _cdecl	DeleteProc		(CProgressDlg *pDlg);
	static	int  _cdecl	InsertProc		(CProgressDlg *pDlg);
	static	int  _cdecl	ExtractProc		(CProgressDlg *pDlg);
	static	void _cdecl	ProgressProc	(CProgressDlg *pDlg);

	static	void _cdecl SelBreak		(void *pParam);
	static	void _cdecl FtpBreak		(void *pParam);
	static	void _cdecl FtpBreakDirect	(void *pParam);
	static	void _cdecl FindBreak		(void *pParam);
	static  UINT _cdecl ShellProc		(void *pParam);
	static  UINT _cdecl StartProc		(void *pParam);
	static  UINT _cdecl ListProc		(void *pParam);
	static  UINT _cdecl ConnectProc     (void *pParam);
	static	UINT _cdecl UpdateProc		(void *pParam);

	static	int  _cdecl CopyRead		(void *pThis,      void *pBuffer,int iBytes);
	static	int  _cdecl CopyWrite		(void *pThis,const void *pBuffer,int iBytes);
	static  CopyStream *CopyCreate		(int iBufferSize=0x10000);
	static  void  		AttrToText		(LPTSTR pText,int iAttr);
	static DWORD WINAPI CopyThread		(LPVOID pParam);
	static DWORD		m_uCopyResult;				// GetLastError vom CopyThread 


};


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline	LPCTSTR	CFtpWindow::GetShortPath    (){return m_cShortPath ;}
inline	LPCTSTR CFtpWindow::GetLastConnError(){return m_pLastError ;}
inline	DWORD   CFtpWindow::GetLastConnErrId(){return m_dwLastError;}
inline	DWORD	CFtpWindow::GetFlags		(){return m_uFlags     ;}

#endif
