//*****************************************************************************
//*
//*
//*		ShellContextMenu.cpp
//*
//*
//*****************************************************************************
#include	"StdAfx.h"
#include	"ShellContextMenu.h"
#include	<malloc.h>


#ifdef		_DEBUG
#undef		 THIS_FILE
static char  THIS_FILE[]=__FILE__;
#define		 new DEBUG_NEW
#endif

#pragma		 comment(lib,"shell32.lib")


#if _MFC_VER <= 0x0699

typedef			ITEMIDLIST	ITEMID_CHILD;
typedef			ITEMIDLIST	ITEMIDLIST_ABSOLUTE;
typedef const	ITEMIDLIST_ABSOLUTE *PCIDLIST_ABSOLUTE;
typedef const	ITEMID_CHILD *PCUITEMID_CHILD;
typedef HRESULT (__stdcall *SHBindToParentType)(PCIDLIST_ABSOLUTE,REFIID,void**,PCUITEMID_CHILD*);


//*****************************************************************************
//*
//*		SHBindToParent_
//*
//*****************************************************************************
static HRESULT SHBindToParent_(PCIDLIST_ABSOLUTE pIdl,REFIID pRiId,void **pPvPtr,PCUITEMID_CHILD *pIdlLast)
{
static SHBindToParentType pProc;
static HMODULE	hDll;


	if(!pProc)
		{
		if(!hDll)hDll = GetModuleHandle("shell32");
		if(!hDll)return 0x80000000;

			pProc = (SHBindToParentType)GetProcAddress(hDll,"SHBindToParent");
		if(!pProc)return 0x80000001;
		}


return pProc(pIdl,pRiId,pPvPtr,pIdlLast);
}

#define		SHBindToParent				SHBindToParent_
#endif





#define		MIN_ID						1
#define		MAX_ID						10000

static		IContextMenu2  *pIContext2 = NULL;
static		IContextMenu3  *pIContext3 = NULL;

//*****************************************************************************
//*
//*		CShellContextMenu
//*
//*****************************************************************************
CShellContextMenu::CShellContextMenu()
{

	m_pFolder		= NULL;
	m_pArray		= NULL;
	m_pMenu			= NULL;
	m_pExecute		= NULL;
	m_pExecParam	= NULL;
	m_uLock			= 0;

}

//*****************************************************************************
//*
//*		CShellContextMenu
//*
//*****************************************************************************
CShellContextMenu::~CShellContextMenu()
{

	if(m_pFolder && m_bDelete)									// free all allocated datas
		{
		m_pFolder->Release();
		}

	m_pFolder = NULL;
	FreePIDLArray(m_pArray);
	m_pArray = NULL;

	if(m_pMenu)
		{
		m_pMenu->DestroyMenu();
		delete m_pMenu;
		m_pMenu = NULL;
		}

}



//*****************************************************************************
//*
//*		GetContextMenu
//*
//*****************************************************************************
//  this functions determines which version of IContextMenu is avaibale for
//  those objects (always the highest one) and returns that interface
BOOL CShellContextMenu::GetContextMenu(void **pContextMenu,int &iMenuType)
{
static OSVERSIONINFO	sInfo;	
LPCONTEXTMENU			pIcm1;



	if(sInfo.dwMajorVersion==0)									// get OS version
		{
		sInfo.dwOSVersionInfoSize=sizeof(sInfo);
		GetVersionEx(&sInfo);
		}

	*pContextMenu	= NULL;
	pIcm1			= NULL;

																// first we retrieve the normal IContextMenu interface (every object should have it)
	m_pFolder->GetUIObjectOf(NULL,m_iItems,(LPCITEMIDLIST*)m_pArray,IID_IContextMenu,NULL,(void**)&pIcm1);

	if(!pIcm1)													// since we got an IContextMenu interface we can now obtain the higher version interfaces via that
		{
		return FALSE; 											// something went wrong
		}

	if(sInfo.dwMajorVersion<=5)									// has a BUG on Win7
		{
		if(pIcm1->QueryInterface(IID_IContextMenu3,pContextMenu)==NOERROR)
			{
			iMenuType = 3;
			}
		else if(pIcm1->QueryInterface(IID_IContextMenu2,pContextMenu)==NOERROR)
			{
			iMenuType = 2;
			}
		}

	if(*pContextMenu)
		{
		pIcm1->Release();										// we can now release version 1 interface, cause we got a higher one
		}
	else{
		iMenuType		= 1;
		*pContextMenu	= pIcm1; 								// since no higher versions were found
		}														// redirect pContextMenu to version 1 interface


return TRUE;
}


//*****************************************************************************
//*
//*		HookWndProc
//*
//*****************************************************************************
LRESULT CALLBACK CShellContextMenu::HookWndProc(HWND hWnd,UINT uMessage,WPARAM wParam,LPARAM lParam)
{
LRESULT		lResult;


	switch(uMessage)
		{
	case WM_MENUCHAR:											// only supported by IContextMenu3

		if(pIContext3)
			{
			lResult = 0;
			pIContext3->HandleMenuMsg2(uMessage,wParam,lParam,&lResult);
			return lResult;
			}

		break;

	case WM_DRAWITEM:
	case WM_MEASUREITEM:
		
		if(wParam)
			{
			break;												// if wParam != 0 then the uMessage is not menu-related
			}

	case WM_INITMENUPOPUP:

		if(pIContext2)
			{
			pIContext2->HandleMenuMsg(uMessage,wParam,lParam);
			}
		else{													// version 3
			pIContext3->HandleMenuMsg(uMessage,wParam,lParam);
			}

		return (uMessage==WM_INITMENUPOPUP)? 0:1;				// inform caller that we handled WM_INITPOPUPMENU by ourself

	default:

		break;
		}

	

return ::CallWindowProc((WNDPROC)GetProp(hWnd,TEXT("OldWndProc")),hWnd,uMessage,wParam,lParam);
}


//*****************************************************************************
//*
//*		ShowContextMenu
//*
//*****************************************************************************
UINT CShellContextMenu::ShowContextMenu(CWnd *pWnd,CPoint sPoint)
{
LPCONTEXTMENU	pContextMenu;
WNDPROC			pOldWndProc;
UINT			uCommand;
int				iMenuType;
LRESULT			lRet;


	iMenuType = 0;

	if(!GetContextMenu((void**)&pContextMenu,iMenuType))
		{
		return 0; 												// something went wrong
		}

	if(!m_pMenu)
		{
		m_pMenu = new CMenu;
		m_pMenu->CreatePopupMenu();
		}

																// lets fill the our popupmenu
	pContextMenu->QueryContextMenu(m_pMenu->m_hMenu,m_pMenu->GetMenuItemCount(),MIN_ID,MAX_ID,CMF_NORMAL|CMF_EXPLORE);

	if(m_uLock)
		{
		CHAR			cText[128];
		MENUITEMINFO	sInfo;
		BOOL			bFlag;
		int				iCount;
		int				iPos;
		
		iCount = GetMenuItemCount(m_pMenu->m_hMenu);

		
		for(iPos=0;iPos<iCount;iPos++)
			{
			sInfo.cbSize	 = sizeof(sInfo);
			sInfo.fMask		 = MIIM_DATA|MIIM_TYPE|MIIM_ID;
			sInfo.dwTypeData = cText;
			sInfo.cch		 = 128;
			cText[0]		 = 0;

			if(!GetMenuItemInfo(m_pMenu->m_hMenu,iPos,TRUE,&sInfo))continue;
			if(!cText[0]       )continue;
			if(sInfo.wID>MAX_ID)continue;
			if(sInfo.wID<MIN_ID)continue;

			bFlag = FALSE;
			
			if(m_uLock&0x01)
				{
				if(sInfo.cch==8 && !_tcsicmp(cText,_T("&Lschen")))bFlag=TRUE;
				if(sInfo.cch==7 && !_tcsicmp(cText,_T( "Lschen")))bFlag=TRUE;
				if(sInfo.cch==7 && !_tcsicmp(cText,_T("&Delete" )))bFlag=TRUE;
				if(sInfo.cch==6 && !_tcsicmp(cText,_T( "Delete" )))bFlag=TRUE;
				}

			if(m_uLock&0x02)
				{
				if(sInfo.cch==9 && !_tcsicmp(cText,_T("&Kopieren")))bFlag=TRUE;
				if(sInfo.cch==8 && !_tcsicmp(cText,_T( "Kopieren")))bFlag=TRUE;
				if(sInfo.cch==5 && !_tcsicmp(cText,_T("&Copy"    )))bFlag=TRUE;
				if(sInfo.cch==4 && !_tcsicmp(cText,_T( "Copy"    )))bFlag=TRUE;
				}

			if(m_uLock&0x04)
				{
				if(sInfo.cch==13 && !_tcsicmp(cText,_T("&Ausschneiden")))bFlag=TRUE;
				if(sInfo.cch==12 && !_tcsicmp(cText,_T( "Ausschneiden")))bFlag=TRUE;
				if(sInfo.cch==3  && !_tcsicmp(cText,_T( "Cut"         )))bFlag=TRUE;
				}

			if(m_uLock&0x08)
				{
				if(sInfo.cch==9 && !_tcsicmp(cText,_T("&Einfgen")))bFlag=TRUE;
				if(sInfo.cch==8 && !_tcsicmp(cText,_T( "Einfgen")))bFlag=TRUE;
				if(sInfo.cch==7 && !_tcsicmp(cText,_T("&Insert"  )))bFlag=TRUE;
				if(sInfo.cch==6 && !_tcsicmp(cText,_T( "Insert"  )))bFlag=TRUE;
				}

			if(m_uLock&0x10)
				{										 
				if(sInfo.cch==22 && !_tcsicmp(cText,_T("&Verknpfung erstellen")))bFlag=TRUE;
				if(sInfo.cch==21 && !_tcsicmp(cText,_T( "Verknpfung erstellen")))bFlag=TRUE;
				}

			if(bFlag)
				{
				EnableMenuItem(m_pMenu->m_hMenu,iPos,MF_BYPOSITION|MF_GRAYED);
				}
			}
		}
																// subclass window to handle menurelated messages in CShellContextMenu

	if(iMenuType>1)												// only subclass if its version 2 or 3
		{
		pOldWndProc = (WNDPROC)SetWindowLong(pWnd->m_hWnd,GWL_WNDPROC,(DWORD)HookWndProc);

		if(iMenuType==2)
			{
			pIContext2 = (LPCONTEXTMENU2)pContextMenu;
			}
		else{
			pIContext3 = (LPCONTEXTMENU3)pContextMenu;			// version 3
			}
		}
	else{
		pOldWndProc = NULL;
		}


	uCommand = m_pMenu->TrackPopupMenu(TPM_RETURNCMD|TPM_LEFTALIGN,sPoint.x,sPoint.y,pWnd);
	

	if(pOldWndProc)												// unsubclass
		{
		SetWindowLong(pWnd->m_hWnd,GWL_WNDPROC,(DWORD)pOldWndProc);
		}

	if(uCommand>=MIN_ID && uCommand<=MAX_ID)					// see if returned uCommand belongs to shell menu entries
		{
		lRet = 1;

		if(m_pExecute)
			{
			lRet = m_pExecute(pWnd->m_hWnd,m_pExecParam,uCommand);
			}

		if(lRet)
			{
			InvokeCommand(pContextMenu,uCommand-MIN_ID);		// execute related command
			uCommand = 0;
			}
		}


	pContextMenu->Release();
	pIContext2 = NULL;
	pIContext3 = NULL;



return uCommand;
}


//*****************************************************************************
//*
//*		InvokeCommand
//*
//*****************************************************************************
void CShellContextMenu::InvokeCommand(LPCONTEXTMENU pContextMenu,UINT uCommand)
{
CMINVOKECOMMANDINFO		sCmi={0};


	sCmi.cbSize	=  sizeof(CMINVOKECOMMANDINFO);
	sCmi.lpVerb	= (LPSTR)MAKEINTRESOURCE(uCommand);
	sCmi.nShow	=  SW_SHOWNORMAL;

	pContextMenu->InvokeCommand(&sCmi);
}

//*****************************************************************************
//*
//*		SetCallback
//*
//*****************************************************************************
void CShellContextMenu::SetCallback(LPVOID pParam,ExecuteCallback pProc)
{

	m_pExecParam = pParam;
	m_pExecute   = pProc;

}

//*****************************************************************************
//*
//*		SetObjects
//*
//*****************************************************************************
void CShellContextMenu::SetObjects(LPCTSTR pStrObject)
{
CStringArray	sArray;


	sArray.Add(pStrObject);									// create a CStringArray with one element
	SetObjects(sArray);										// and pass it to SetObjects(CStringArray &sArray)

}


//*****************************************************************************
//*
//*		SetObjects
//*
//*****************************************************************************
void CShellContextMenu::SetObjects(CStringArray &sArray)
{
IShellFolder   *pDesktop;
IShellFolder   *pFolder;
LPMALLOC		pMalloc;
LPITEMIDLIST	pIdlItem;
LPITEMIDLIST	pIdl;
int				i;


	if(m_pFolder && m_bDelete)								// free all allocated datas
		{
		m_pFolder->Release();
		}

	FreePIDLArray(m_pArray);

	m_pArray  = NULL;
	m_pFolder = NULL;
	pDesktop  = NULL;
	pIdl	  = NULL;

	SHGetDesktopFolder(&pDesktop);							// get IShellFolder interface of Desktop (root of shell namespace)

	// ParseDisplayName creates a PIDL from a file system path relative to the IShellFolder interface
	// but since we use the Desktop as our interface and the Desktop is the namespace root
	// that means that it's a fully qualified PIDL, which is what we need


	#ifndef		_UNICODE

	OLECHAR	   *pOlePath=NULL;

	pOlePath = (OLECHAR*)calloc(sArray.GetAt(0).GetLength()+16,sizeof(OLECHAR));

	MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,sArray.GetAt(0).GetBuffer(0),-1,pOlePath,sArray.GetAt(0).GetLength()+1);
	pDesktop->ParseDisplayName(NULL,0,pOlePath,NULL,&pIdl,NULL);

	free(pOlePath);

	#else

	pDesktop->ParseDisplayName(NULL,0,sArray.GetAt(0).GetBuffer(0),NULL,&pIdl,NULL);

	#endif

	if(!pIdl)
		{
		pDesktop->Release();
		return;
		}

															// now we need the parent IShellFolder interface of pIdl, and the relative PIDL to that interface
	pIdlItem = NULL;										// relative pIdl
	ShBindToParentEx(pIdl,IID_IShellFolder,(void**)&m_pFolder,NULL);
	free(pIdlItem);


	if(!m_pFolder)
		{
		pDesktop->Release();
		return;
		}

	pMalloc = NULL;											// get interface to IMalloc (need to free the PIDLs allocated by the shell functions)
	SHGetMalloc(&pMalloc);
	pMalloc->Free(pIdl);

	// now we have the IShellFolder interface to the parent folder specified in the first element in sArray
	// since we assume that all objects are in the same folder (as it's stated in the MSDN)
	// we now have the IShellFolder interface to every objects parent folder

	pFolder	 = NULL;
	m_iItems = sArray.GetSize();
	m_pArray = (LPITEMIDLIST*)calloc(1,(m_iItems+2)*sizeof(LPITEMIDLIST));

	for(i=0;i<m_iItems;i++)
		{
		#ifndef		_UNICODE

		pOlePath = (OLECHAR*)calloc(sArray.GetAt(i).GetLength()+4,sizeof(OLECHAR));
		MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,sArray.GetAt(i).GetBuffer(0),-1,pOlePath,sArray.GetAt(i).GetLength()+1);
		pDesktop->ParseDisplayName(NULL,0,pOlePath,NULL,&pIdl,NULL);
		free(pOlePath);

		#else

		pDesktop->ParseDisplayName(NULL,0,sArray.GetAt(i).GetBuffer(0),NULL,&pIdl,NULL);

		#endif
															// get relative pIdl via SHBindToParent
		ShBindToParentEx(pIdl,IID_IShellFolder,(void**)&pFolder,(LPCITEMIDLIST*)&pIdlItem);


		if(pIdlItem)
			{
			m_pArray[i] = CopyPIDL(pIdlItem);				// copy relative pIdl to pArray
			free(pIdlItem);
			}
		else{
			m_pArray[i] = NULL;
			}

		pMalloc->Free(pIdl);								// free pIdl allocated by ParseDisplayName
		if(pFolder)pFolder->Release();
		}

	pMalloc ->Release();
	pDesktop->Release();

	m_bDelete = TRUE;

}


//*****************************************************************************
//*
//*		SetObjects
//*
//*****************************************************************************
//  only one full qualified PIDL has been passed
void CShellContextMenu::SetObjects(LPITEMIDLIST pIdl)
{
LPITEMIDLIST	pIdlItem;
LPMALLOC		pMalloc;



	if(m_pFolder && m_bDelete)								// free all allocated datas
		{
		m_pFolder->Release ();
		}


	m_pFolder = NULL;
	FreePIDLArray (m_pArray);
	m_pArray  = NULL;

															// full qualified PIDL is passed so we need
															// its parent IShellFolder interface and its relative PIDL to that
	pIdlItem = NULL;
	SHBindToParent((LPCITEMIDLIST)pIdl,IID_IShellFolder,(void**)&m_pFolder,(LPCITEMIDLIST*)&pIdlItem);

	m_pArray 	= (LPITEMIDLIST *)calloc(1,sizeof(LPITEMIDLIST)*2);	// allocate ony for one elemnt
	m_pArray[0]	=  CopyPIDL (pIdlItem);



	pMalloc = NULL;											// now free pIdlItem via IMalloc interface (but not m_pFolder, that we need later
	SHGetMalloc(&pMalloc);
	pMalloc->Free(pIdlItem);
	pMalloc->Release();

	m_iItems	= 1;
	m_bDelete	= TRUE; 									// indicates that m_pFolder should be deleted by CShellContextMenu
}


//*****************************************************************************
//*
//*		SetObjects
//*
//*****************************************************************************
//  IShellFolder interface with a relative pIdl has been passed
void CShellContextMenu::SetObjects(IShellFolder *pFolder,LPITEMIDLIST pIdlItem)
{

	if(m_pFolder && m_bDelete)								// free all allocated datas
		{
		m_pFolder->Release ();
		}

	m_pFolder	= NULL;
	FreePIDLArray (m_pArray);
	m_pArray	= NULL;
	m_pFolder	= pFolder;
	m_pArray 	= (LPITEMIDLIST*)calloc(1,sizeof(LPITEMIDLIST)*2);
	m_pArray[0]	=  CopyPIDL (pIdlItem);
	m_iItems	=  1;
	m_bDelete 	=  FALSE;									// indicates wheter m_pFolder should be deleted by CShellContextMenu

}

//*****************************************************************************
//*
//*		SetObjects
//*
//*****************************************************************************
void CShellContextMenu::SetObjects(IShellFolder *pFolder,LPITEMIDLIST *pArray,int iItemCount)
{
int		i;


	if(m_pFolder && m_bDelete)									// free all allocated datas
		{
		m_pFolder->Release ();
		}


	m_pFolder = NULL;
	FreePIDLArray(m_pArray);
	m_pArray  = NULL;
	m_pFolder = pFolder;
	m_pArray  = (LPITEMIDLIST*)calloc(1,(iItemCount*2)*sizeof(LPITEMIDLIST));


	for(i=0;i<iItemCount;i++)
		{
		m_pArray[i] = CopyPIDL(pArray[i]);
		}


	m_iItems  = iItemCount;
	m_bDelete = FALSE;											// indicates wheter m_pFolder should be deleted by CShellContextMenu
}


//*****************************************************************************
//*
//*		FreePIDLArray
//*
//*****************************************************************************
void CShellContextMenu::FreePIDLArray(LPITEMIDLIST *pArray)
{
int		iSize,i;


	if(!pArray)
		{
		return;
		}

	iSize = _msize(pArray)/sizeof(LPITEMIDLIST);

	for(i=0;i<iSize;i++)
		{
		if(!pArray[i])continue;
		free(pArray[i]);
		}

	free(pArray);

}


//*****************************************************************************
//*
//*		CopyPIDL
//*
//*****************************************************************************
LPITEMIDLIST CShellContextMenu::CopyPIDL(LPCITEMIDLIST pIdl,int iBytes)
{
LPITEMIDLIST	pIdlRet;


	if(iBytes == -1)
		{
		iBytes = GetPIDLSize(pIdl);								// Calculate size of list.
		}

	   pIdlRet = (LPITEMIDLIST)calloc(iBytes+16,sizeof(BYTE));
	if(pIdlRet)
		{
		CopyMemory(pIdlRet,pIdl,iBytes);
		}


return pIdlRet;
}


//*****************************************************************************
//*
//*		GetPIDLSize
//*
//*****************************************************************************
UINT CShellContextMenu::GetPIDLSize(LPCITEMIDLIST pIdl)
{
LPITEMIDLIST	pIdlTemp;
int				iSize;

	if(!pIdl)
		{
		return 0;
		}

	iSize		= 0;
	pIdlTemp	= (LPITEMIDLIST)pIdl;

	while(pIdlTemp->mkid.cb)
		{
		iSize	   +=  pIdlTemp->mkid.cb;
		pIdlTemp	= (LPITEMIDLIST)(((LPBYTE)pIdlTemp)+pIdlTemp->mkid.cb);
		}

return iSize;
}

//*****************************************************************************
//*
//*		GetMenu
//*
//*****************************************************************************
CMenu *CShellContextMenu::GetMenu()
{

	if(!m_pMenu)
		{
		m_pMenu = new CMenu;
		m_pMenu->CreatePopupMenu();								// create the popupmenu (its empty)
		}


return m_pMenu;
}


//*****************************************************************************
//*
//*		ShBindToParentEx
//*
//*****************************************************************************
//  this is workaround function for the Shell API Function SHBindToParent
//  SHBindToParent is not available under Win95/98
HRESULT CShellContextMenu::ShBindToParentEx (LPCITEMIDLIST pIdl,REFIID pRiId,VOID **pPvPtr,LPCITEMIDLIST *pIdlLast)
{
IShellFolder   *pFolder;
IShellFolder   *pDesktop;
LPITEMIDLIST	pIdlParent;
LPBYTE			pRel;
HRESULT			uRes;
int				iCnt;


	if(!pIdl || !pPvPtr)
		{
		return E_POINTER;
		}

	iCnt	= GetPIDLCount(pIdl);
	uRes	= 0;

	if(iCnt==0) 											// desktop pIdl of invalid pIdl
		{
		return E_POINTER;
		}


	pDesktop = NULL;
	SHGetDesktopFolder(&pDesktop);


	if(iCnt==1) 											// desktop pIdl
		{
		   uRes = pDesktop->QueryInterface(pRiId, pPvPtr);
		if(uRes==S_OK)
			{
			if(pIdlLast)
				{
				*pIdlLast = CopyPIDL(pIdl);
				}
			}

		pDesktop->Release();

		return uRes;
		}


	pFolder		= NULL;
	pIdlParent	= NULL;
	pRel		= GetPIDLPos(pIdl,iCnt-1);
	pIdlParent	= CopyPIDL(pIdl,pRel-(LPBYTE)pIdl);
	uRes		= pDesktop->BindToObject(pIdlParent,NULL,__uuidof(pFolder),(void **)&pFolder);


	if(uRes!=S_OK)
		{
		free(pIdlParent);
		pDesktop->Release();

		return uRes;
		}

	   uRes = pFolder->QueryInterface(pRiId,pPvPtr);
	if(uRes==S_OK)
		{
		if(pIdlLast)
			{
			*pIdlLast = CopyPIDL((LPCITEMIDLIST)pRel);
			}
		}

	free(pIdlParent);

	pFolder ->Release();
	pDesktop->Release();


return uRes;
}


//*****************************************************************************
//*
//*		GetPIDLPos
//*
//*****************************************************************************
LPBYTE CShellContextMenu::GetPIDLPos (LPCITEMIDLIST pIdl,int iPos)
{
LPBYTE		pCur;
int			iCnt;


	if(!pIdl)
		{
		return 0;
		}

	iCnt = 0;
	pCur = (LPBYTE)pIdl;

	while(((LPCITEMIDLIST)pCur)->mkid.cb)
		{
		if(iCnt==iPos)
			{
			return pCur;
			}

		iCnt++;
		pCur += ((LPCITEMIDLIST) pCur)->mkid.cb;				// + sizeof(pIdl->mkid.cb);
		}

	if(iCnt==iPos)
		{
		return pCur;
		}


return NULL;
}


//*****************************************************************************
//*
//*		GetPIDLCount
//*
//*****************************************************************************
int CShellContextMenu::GetPIDLCount(LPCITEMIDLIST pIdl)
{
LPBYTE		pCur;
int			iCnt;


	if(!pIdl)
		{
		return 0;
		}

	iCnt = 0;
	pCur = (LPBYTE)pIdl;

	while(((LPCITEMIDLIST)pCur)->mkid.cb)
		{
		iCnt += 1;
		pCur += ((LPCITEMIDLIST)pCur)->mkid.cb;
		}


return iCnt;
}

//*****************************************************************************
//*
//*		SetLock
//*
//*****************************************************************************
//	Stellt ein welche Mens gesperrt werden sollen
void CShellContextMenu::SetLock(UINT uBits)
{

	m_uLock = uBits;

}